@@ -1,7 +1,7 @@
 /* Display inputs for a new bank name when collecting new bank account info.
  */
 
-import { displayInput } from './modules/display_new_choice_form_input.js';
+import { AcquisitionFormManager } from './modules/manage_acquisition_form.js';
 
 
 (function() {
@@ -9,7 +9,8 @@ import { displayInput } from './modules/display_new_choice_form_input.js';
   // Display the bank name input box for the new bank
   let $inputBank = $('form#bank-account #bank_info-bank_id');
   let $fieldBankName = $('form#bank-account #bank-name-field');
-  displayInput(
+
+  const bankAcquisitionFormManager = new AcquisitionFormManager(
     $inputBank,
     $fieldBankName,
   );
